###############################################################################
##
## Copyright (c) 2006 Xilinx, Inc. All Rights Reserved.
##
## xps_ethernetlite_v2_1_0.tcl
##
###############################################################################

## @BEGIN_CHANGELOG EDK_J
##
## - initial 1.00a version
##
## @END_CHANGELOG
##

#***--------------------------------***-----------------------------------***
#
#			     CORE_LEVEL_CONSTRAINTS
#
#***--------------------------------***-----------------------------------***

proc generate_corelevel_ucf {mhsinst} {

    set  filePath [xget_ncf_dir $mhsinst]

    file mkdir    $filePath

    # specify file name
    set    instname   [xget_hw_parameter_value $mhsinst "INSTANCE"]
    set    name_lower [string 	  tolower    $instname]
    set    fileName   $name_lower
    append fileName   "_wrapper.ucf"
    append filePath   $fileName

    set    duplex     [xget_value $mhsinst   "parameter" "C_DUPLEX"]

    # Open a file for writing
    set    outputFile [open $filePath "w"]

    # Send a line to the output file
    puts   $outputFile "NET \"PHY_rx_clk\" TNM_NET = \"RXCLK_GRP_$instname\";"
    puts   $outputFile "NET \"PHY_tx_clk\" TNM_NET = \"TXCLK_GRP_$instname\";"
    puts   $outputFile "TIMESPEC \"TSTXOUT_$instname\" = FROM \"TXCLK_GRP_$instname\" TO \"PADS\" 10 ns;"
    puts   $outputFile "TIMESPEC \"TSRXIN_$instname\" = FROM \"PADS\" TO \"RXCLK_GRP_$instname\" 6 ns;"

    set target_family [ xget_hw_parameter_value $mhsinst "C_FAMILY" ]

    # family based constraints
    # if virtex family
    if { [xcheck_virtex_device $target_family] == 1 } {

    	puts   $outputFile "NET \"PHY_rx_clk\" USELOWSKEWLINES;"
    	puts   $outputFile "NET \"PHY_tx_clk\" USELOWSKEWLINES;"

    }

    # for devices V4
    if { [xcheck_virtex4_device $target_family] == 1 } {

    	puts   $outputFile "NET \"PHY_tx_clk\" MAXSKEW= 1.0 ns;"
    	puts   $outputFile "NET \"PHY_rx_clk\" MAXSKEW= 1.0 ns;"

    } else {
    
    	puts   $outputFile "NET \"PHY_tx_clk\" MAXSKEW= 2.0 ns;"
    	puts   $outputFile "NET \"PHY_rx_clk\" MAXSKEW= 2.0 ns;"
    
    }

    puts   $outputFile "NET \"PHY_rx_clk\" PERIOD = 40 ns HIGH 14 ns;"
    puts   $outputFile "NET \"PHY_tx_clk\" PERIOD = 40 ns HIGH 14 ns;"

    # if C_FAMILY == V4, V5 or SPARTAN3E, use IOBDELAY=NONE and 
    # NODELAY for everything else
    set delay "NODELAY"

    if { [xcheck_virtex4_device $target_family] == 1 || [string compare -nocase $target_family "virtex5"] == 0 || [string compare -nocase $target_family "spartan3e"] == 0 } {

    	set delay "IOBDELAY=NONE"

    } 

    puts   $outputFile "NET \"PHY_rx_data<3>\" $delay;"
    puts   $outputFile "NET \"PHY_rx_data<2>\" $delay;"
    puts   $outputFile "NET \"PHY_rx_data<1>\" $delay;"
    puts   $outputFile "NET \"PHY_rx_data<0>\" $delay;"
    puts   $outputFile "NET \"PHY_dv\" $delay;"
    puts   $outputFile "NET \"PHY_rx_er\" $delay;"
    puts   $outputFile "NET \"PHY_crs\" $delay;"

    if { $duplex == 0 } {

        puts   $outputFile "NET \"PHY_col\" $delay;"
    }

    # Close the file
    close  $outputFile

    puts   [xget_ncf_loc_info $mhsinst]

}
