-------------------------------------------------------------------------------
-- $Id: transmit_2.vhd,v 1.1 2007/12/11 06:41:15 sanjayk Exp $
-------------------------------------------------------------------------------
-- transmit - entity/architecture pair
-------------------------------------------------------------------------------
--  ***************************************************************************
--  **  Copyright(C) 2006 by Xilinx, Inc. All rights reserved.               **
--  **                                                                       **
--  **  This text contains proprietary, confidential                         **
--  **  information of Xilinx, Inc. , is distributed by                      **
--  **  under license from Xilinx, Inc., and may be used,                    **
--  **  copied and/or disclosed only pursuant to the terms                   **
--  **  of a valid license agreement with Xilinx, Inc.                       **
--  **                                                                       **
--  **  Unmodified source code is guaranteed to place and route,             **
--  **  function and run at speed according to the datasheet                 **
--  **  specification. Source code is provided "as-is", with no              **
--  **  obligation on the part of Xilinx to provide support.                 **
--  **                                                                       **
--  **  Xilinx Hotline support of source code IP shall only include          **
--  **  standard level Xilinx Hotline support, and will only address         **
--  **  issues and questions related to the standard released Netlist        **
--  **  version of the core (and thus indirectly, the original core source). **
--  **                                                                       **
--  **  The Xilinx Support Hotline does not have access to source            **
--  **  code and therefore cannot answer specific questions related          **
--  **  to source HDL. The Xilinx Support Hotline will only be able          **
--  **  to confirm the problem in the Netlist version of the core.           **
--  **                                                                       **
--  **  This copyright and support notice must be retained as part           **
--  **  of this text at all times.                                           **
--  ***************************************************************************
--
-------------------------------------------------------------------------------
-- Filename:        transmit.vhd
--
-- Description:     This is the transmit path portion of the design
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--
--  xps_ethernetlite.vhd
--      \
--      \-- xemac.vhd
--           \
--           \--xps_ipif_ssp1.vhd
--           \
--           \-- emac_dpram.vhd                     
--           \    \                     
--           \    \-- RAMB16_S36_S36
--           \    \     or
--           \    \-- RAMB4_S8_S8
--           \                          
--           \-- emac.vhd                     
--                \                     
--                \                     
--                \-- receive.vhd
--                \      rx_state_2.vhd
--                \      rx_intrfce.vhd
--                \         veasyncfifo5w15ddstrb.edn
--                \            or
--                \         v2asyncfifo5w15ddstrb.edn
--                \      cntr4bit.vhd
--                \      nibshiftin.vhd
--                \      nibshiftdain.vhd
--                \      cntr11bit.vhd
--                \      cntr12bit.vhd
--                \      dacheck.vhd
--                \      rxlengthcntr.vhd
--                \      crcgenrx.vhd
--                \      rxcrcnibcnt.vhd
--                \                     
--                \-- transmit_2.vhd
--                       crcgentx.vhd
--                          crcnibshiftreg
--                       tx_intrfce.vhd
--                          veasyncfifo5w15ddstrb.edn
--                             or
--                          v2asyncfifo5w15ddstrb.edn
--                       cntr11bit.vhd
--                       nibshiftout.vhd
--                       tx_state_3.vhd
--                       deferral.vhd
--                          cntr5bit.vhd
--                          defer_state.vhd
--                       bocntr.vhd
--                          lfsr16.vhd
--                       msh_cnt.vhd
--                       ld_arith_reg.vhd
--
-------------------------------------------------------------------------------
-- Author:         MSH
-- History:
--  RSK            11/06/06
-- ^^^^^^
--                 First Version, Based on opb_ethernetlite v1.01.b
--  ~~~~~
--  RSK            12/15/06
-- ^^^^^^
--                 RTL bus Mux is replaced by mux_onehot_f (proc_common_v2_00_a)
--                 for signal emac_tx_wr_data
--  ~~~~~
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------
-- 
library ieee;
use ieee.STD_LOGIC_1164.all;

--Package file that contains constant definition for RESET_ACTIVE
library xps_ethernetlite_v2_00_c;
use xps_ethernetlite_v2_00_c.mac_pkg.all;

library proc_common_v2_00_a;
use proc_common_v2_00_a.all;
-- synopsys translate_off
Library XilinxCoreLib;
library simprim;
-- synopsys translate_on
library unisim;
use unisim.Vcomponents.all;

-------------------------------------------------------------------------------
-- Port Declaration
-------------------------------------------------------------------------------
entity transmit is
  generic 
           (
            C_DUPLEX : integer := 1; -- 1 = full duplex, 0 = half duplex   
            C_FAMILY : string  := "virtex5"  
           );
  port 
   ( 
    nibbleLength         : in std_logic_vector(0 to 11);
    txClkEn              : in std_logic;
    phy_tx_clk           : in std_logic;
    phy_crs              : in std_logic;
    phy_col              : in std_logic;
    
    phy_tx_en            : out std_logic;
    phy_tx_data          : out std_logic_vector (0 to 3);
    tx_addr_load         : out std_logic;
    tx_addr_en           : out std_logic;
    tx_start             : out std_logic;
    tx_done              : out std_logic;
    tx_done_irq          : out std_logic;
    tx_pong_ping_l       : in  std_logic;
    Tx_idle              : out std_logic;
    enbl_txLength_nib1   : out std_logic;
    enbl_txLength_nib2   : out std_logic;
    enbl_txLength_nib3   : out std_logic;
    enbl_txLength_nib4   : out std_logic;
    lengthDelay1         : out std_logic;
    lengthDelay2         : out std_logic;
    tx_DPM_ce            : out std_logic;
    tx_DPM_wr_data       : out std_logic_vector (0 to 3);
    tx_DPM_rd_data       : in std_logic_vector (0 to 3);
    tx_DPM_wr_rd_n       : out std_logic;
    
    mac_addr_ram_we      : out std_logic;
    mac_addr_ram_addr_wr : out std_logic_vector(0 to 3);    
        
    rst                  : in std_logic;
    clk                  : in std_logic
    );
end transmit;

-------------------------------------------------------------------------------
-- Definition of Generics:
--          No Generics were used for this Entity.
--
-- Definition of Ports:
--         
-------------------------------------------------------------------------------

architecture imp of transmit is

-------------------------------------------------------------------------------
--  Constant Declarations
-------------------------------------------------------------------------------
constant logic_one               :std_logic := '1';

-------------------------------------------------------------------------------
--  Signal and Type Declarations
-------------------------------------------------------------------------------
signal full_half_n               : std_logic;
signal bus_combo                 : std_logic_vector (0 to 5);
signal txChannelReset            : std_logic;
signal emac_tx_wr                : std_logic;
signal txfifo_full               : std_logic;
signal txfifo_empty              : std_logic;
signal tx_en                     : std_logic;
signal fifo_tx_en                : std_logic;
signal txNibbleCntLd             : std_logic;
signal txNibbleCntEn             : std_logic;
signal txNibbleCntRst            : std_logic;
signal txComboNibbleCntRst       : std_logic;
signal phy_tx_en_i               : std_logic;
signal deferring                 : std_logic;
signal txBusFifoWrCntRst         : std_logic;
signal txBusFifoWrCntEn          : std_logic;
signal txComboBusFifoWrCntRst    : std_logic;
signal txComboBusFifoWrCntEn     : std_logic;
signal txComboColRetryCntRst_n   : std_logic;
signal txComboBusFifoRst         : std_logic;
signal txColRetryCntRst_n        : std_logic;
signal enblPreamble              : std_logic;
signal enblSFD                   : std_logic;
signal enblData                  : std_logic;
signal enblJam                   : std_logic;
signal enblCRC                   : std_logic;
signal txCntEnbl                 : std_logic;
signal txColRetryCntEnbl         : std_logic;
signal jamTxNibCntEnbl           : std_logic;
signal jamTxComboNibCntEnbl      : std_logic;
signal txRetryRst                : std_logic;
signal txLateColnRst             : std_logic;
signal initBackoff               : std_logic;
signal backingOff                : std_logic;
signal txCrcShftOutEn            : std_logic;
signal txCrcEn                   : std_logic;
signal crcComboRst               : std_logic;
signal emac_tx_wr_data           : std_logic_vector (0 to 3);
signal crcCnt                    : std_logic_vector(0 to 3);
signal collisionRetryCnt         : std_logic_vector(0 to 4);
signal jamTxNibbleCnt            : std_logic_vector(0 to 3);
signal colWindowNibbleCnt        : std_logic_vector(0 to 7);
signal prb                       : std_logic_vector(0 to 3);
signal Sfd                       : std_logic_vector(0 to 3);
signal jam                       : std_logic_vector(0 to 3);
signal crc                       : std_logic_vector(0 to 3);
signal currentTxBusFifoWrCnt     : std_logic_vector(0 to 11);
signal currentTxNibbleCnt        : std_logic_vector (0 to 11);
signal phy_tx_en_n               : std_logic;
signal txComboColRetryCntRst     : std_logic;
signal phy_tx_en_i_n             : std_logic;
signal jam_rst                   : std_logic;
signal txExcessDefrlRst          : std_logic;
signal enblclear                 : std_logic;
signal tx_en_mod                 : std_logic;
signal emac_tx_wr_mod            : std_logic;
signal pre_sfd_done              : std_logic;
signal mux_in_data :std_logic_vector (0 to 6*4-1);
signal mux_in_sel  :std_logic_vector (0 to 5);

-------------------------------------------------------------------------------
-- Component Declarations
-------------------------------------------------------------------------------
component FDRE
  port 
   (
    Q  : out std_logic;
    C  : in std_logic;
    CE : in std_logic;
    D  : in std_logic;
    R  : in std_logic
   );
end component;

component FDCE
  port 
   (
    Q   : out std_logic;
    C   : in std_logic;
    CE  : in std_logic;
    CLR : in std_logic;
    D   : in std_logic
    );
end component;  

begin

-------------------------------------------------------------------------------
-- tx crc generator
-------------------------------------------------------------------------------
  inst_crcgentx: entity xps_ethernetlite_v2_00_c.crcgentx
    port map 
     (
      rst     => crcComboRst,
      clk     => clk,
      clken   => emac_tx_wr,
      data    => emac_tx_wr_data,
      dataEn  => txCrcEn,
      outEn   => txCrcShftOutEn,
      crcNibs => crc
      );      
              
crcComboRst <= rst or not (fifo_tx_en);

-------------------------------------------------------------------------------
-- tx interface contains the ethernet tx fifo
-------------------------------------------------------------------------------
  inst_tx_intrfce: entity xps_ethernetlite_v2_00_c.tx_intrfce
    port map 
     (     
      phy_tx_clk        => phy_tx_clk,
      emac_tx_wr_data   => emac_tx_wr_data,
      tx_er             => '0',      
      phyTxEn           => tx_en_mod,
      tx_en             => fifo_tx_en,
      fifo_rd_ack       => open,
      fifo_empty        => txfifo_empty,
      fifo_almost_empty => open,
      fifo_full         => txfifo_full,
      emac_tx_wr        => emac_tx_wr_mod,
      phy_tx_data       => bus_combo,
      rst               => txComboBusFifoRst,
      clk               => clk
      );    
              
  txComboBusFifoRst <= rst or txRetryRst or
                       (jam_rst and not(full_half_n) and phy_col and pre_sfd_done);
  
  jam               <= "0110"; -- arbitrary
  prb               <= "0101"; -- transmitted as 1010
  sfd               <= "1101"; -- transmitted as 1011
  

-------------------------------------------------------------------------------
-- PHY output. 
-------------------------------------------------------------------------------
 mux_in_data <= prb & sfd & tx_DPM_rd_data & jam & crc & "0000";
 mux_in_sel  <= enblPreamble & enblSFD & enblData & enblJam & enblCRC & logic_one;

 
 ONR_HOT_MUX:entity proc_common_v2_00_a.mux_onehot_f

   generic map ( 
                 C_DW => 4, 
                 C_NB => 6,
                 C_FAMILY => C_FAMILY
               )

   port map (
             D => mux_in_data, 
             S => mux_in_sel, 
             Y => emac_tx_wr_data
             );


  phy_tx_data <= "0110" when (phy_col = '1' or not(jamTxNibbleCnt(0) = '1' and
                 jamTxNibbleCnt(1) = '0' and jamTxNibbleCnt(2) = '0' and
                 jamTxNibbleCnt(3) = '1')) and full_half_n = '0' and
                 not (jamTxNibbleCnt = "0000") and pre_sfd_done = '1' else
                 
                 "0000" when jamTxNibbleCnt = "0000" and full_half_n = '0' else
                 bus_combo(0 to 3);

  phy_tx_en   <= '1'    when (phy_col = '1' or not(jamTxNibbleCnt(0) = '1' and
                        jamTxNibbleCnt(1) = '0' and jamTxNibbleCnt(2) = '0' and
                        jamTxNibbleCnt(3) = '1')) and full_half_n = '0' and
                        not (jamTxNibbleCnt = "0000") and pre_sfd_done = '1' else 
                 
                 '0'    when jamTxNibbleCnt = "0000" and full_half_n = '0' else 
                 bus_combo(5);
  
-------------------------------------------------------------------------------
-- transmit packet fifo read nibble counter
-------------------------------------------------------------------------------
inst_txNibbleCount: entity proc_common_v2_00_a.ld_arith_reg
  generic  map
    (
    C_ADD_SUB_NOT => false,
    C_REG_WIDTH   => 12,
    C_RESET_VALUE => "000000000000",
    C_LD_WIDTH    => 12,
    C_LD_OFFSET   => 0,
    C_AD_WIDTH    => 12,
    C_AD_OFFSET   => 0
    )
  port map
   (
    CK       => clk,
    RST      => txComboNibbleCntRst,
    Q        => currentTxNibbleCnt,
    LD       => nibbleLength,
    AD       => "000000000001",
    LOAD     => txNibbleCntLd,
    OP       => txNibbleCntEn
    );
    
    txComboNibbleCntRst <= rst or txNibbleCntRst or txRetryRst;
                               
-------------------------------------------------------------------------------
-- transmit state machine
-------------------------------------------------------------------------------
  inst_tx_state_machine: entity xps_ethernetlite_v2_00_c.tx_state_3
    generic map 
      (
      C_DUPLEX             => C_DUPLEX
      )    
    port map
     (
      txClkEn              =>  txClkEn,
      jam_rst              =>  jam_rst,
      txRst                =>  txChannelReset, 
      deferring            =>  deferring,
      colRetryCnt          =>  collisionRetryCnt,
      colWindowNibCnt      =>  colWindowNibbleCnt,
      jamTxNibCnt          =>  jamTxNibbleCnt,
      txNibbleCnt          =>  currentTxNibbleCnt,
      busFifoWrNibbleCnt   =>  currentTxBusFifoWrCnt,
      crcCnt               =>  crcCnt,
      busFifoFull          =>  txfifo_full,
      busFifoEmpty         =>  txfifo_empty,
      phyCollision         =>  phy_col,
      initBackoff          =>  initBackoff,
      txRetryRst           =>  txRetryRst,
      txExcessDefrlRst     =>  txExcessDefrlRst,
      txLateColnRst        =>  txLateColnRst,
      txColRetryCntRst_n   =>  txColRetryCntRst_n,
      txColRetryCntEnbl    =>  txColRetryCntEnbl,
      jamTxNibCntEnbl      =>  jamTxNibCntEnbl,
      txNibbleCntRst       =>  txNibbleCntRst,
      txEnNibbleCnt        =>  txNibbleCntEn, 
      txNibbleCntLd        =>  txNibbleCntLd, 
      busFifoWrCntRst      =>  txBusFifoWrCntRst,
      busFifoWrCntEn       =>  txBusFifoWrCntEn,
      enblPre              =>  enblPreamble,
      enblSFD              =>  enblSFD,
      enblData             =>  enblData,
      enblJam              =>  enblJam,
      enblCRC              =>  enblCRC,
      busFifoWr            =>  emac_tx_wr,
      phytx_en             =>  tx_en, 
      txCrcEn              =>  txCrcEn,
      txCrcShftOutEn       =>  txCrcShftOutEn,
      tx_addr_load         =>  tx_addr_load,        
      tx_addr_en           =>  tx_addr_en,          
      tx_start             =>  tx_start, 
      tx_done              =>  tx_done,
      tx_done_irq          =>  tx_done_irq,
      tx_pong_ping_l       =>  tx_pong_ping_l,
      Tx_idle              =>  Tx_idle,
      enbl_txLength_nib1   =>  enbl_txLength_nib1,  
      enbl_txLength_nib2   =>  enbl_txLength_nib2,  
      enbl_txLength_nib3   =>  enbl_txLength_nib3,  
      enbl_txLength_nib4   =>  enbl_txLength_nib4,  
      lengthDelay1         =>  lengthDelay1,
      lengthDelay2         =>  lengthDelay2,
      tx_DPM_ce            =>  tx_DPM_ce,           
      tx_DPM_wr_data       =>  tx_DPM_wr_data,      
      tx_DPM_rd_data       =>  tx_DPM_rd_data,      
      tx_DPM_wr_rd_n       =>  tx_DPM_wr_rd_n,      
      enblclear            =>  enblclear,
      mac_addr_ram_we      =>  mac_addr_ram_we,     
      mac_addr_ram_addr_wr =>  mac_addr_ram_addr_wr,
      pre_sfd_done         =>  pre_sfd_done,
      clk                  =>  clk,
      rst                  =>  txChannelReset);
       
-------------------------------------------------------------------------------
-- deferral control
-------------------------------------------------------------------------------
  full_half_n <= '1'when C_DUPLEX = 1 else
                 '0';  
                 
  inst_deferral_control: entity xps_ethernetlite_v2_00_c.deferral
    port map
     (
      txEn        => tx_en, 
      txRst       => txChannelReset,
      tx_clk_en   => txClkEn,
      backingOff  => backingOff,
      crs         => phy_crs,
      full_half_n => full_half_n,
      ifgp1       => "10000",
      ifgp2       => "01000",
      deferring   => deferring,
      clk         => clk,
      rst         => rst
      );            
             
-------------------------------------------------------------------------------
-- transmit bus fifo write nibble counter
-------------------------------------------------------------------------------
inst_txBusFifoWriteNibbleCount: entity proc_common_v2_00_a.ld_arith_reg
  generic  map
   (
    C_ADD_SUB_NOT => true,
    C_REG_WIDTH   => 12,
    C_RESET_VALUE => "000000000000",
    C_LD_WIDTH    => 12,
    C_LD_OFFSET   => 0,
    C_AD_WIDTH    => 12,
    C_AD_OFFSET   => 0
    )
  port map
    (
    CK       => clk,
    RST      => txComboBusFifoWrCntRst,
    Q        =>  currentTxBusFifoWrCnt,
    LD       => "000000000000",
    AD       => "000000000001",
    LOAD     => '0',
    OP       => txComboBusFifoWrCntEn
    );
    
  txComboBusFifoWrCntRst <= rst or txBusFifoWrCntRst
                            or txRetryRst;
  txComboBusFifoWrCntEn  <= txBusFifoWrCntEn and emac_tx_wr;

-------------------------------------------------------------------------------
-- crc down counter
-------------------------------------------------------------------------------
phy_tx_en_n <= not(phy_tx_en_i);

inst_crcCounter: entity proc_common_v2_00_a.ld_arith_reg
  generic  map
    (
    C_ADD_SUB_NOT => false,
    C_REG_WIDTH   => 4,
    C_RESET_VALUE => "1000",
    C_LD_WIDTH    => 4,
    C_LD_OFFSET   => 0,
    C_AD_WIDTH    => 4,
    C_AD_OFFSET   => 0
    )
  
  port map
   (
    CK       => clk,
    RST      => txChannelReset,
    Q        =>  crcCnt,
    LD       => "1000",
    AD       => "0001",
    LOAD     => phy_tx_en_n,
    OP       => enblCRC
    );

tx3_NOT_generate: if(C_DUPLEX = 1) generate --Set outputs to zero
  
  begin
  
    collisionRetryCnt  <= (others=> '0');
    colWindowNibbleCnt <= (others=> '0');
    jamTxNibbleCnt     <= (others=> '0');
    backingOff         <= '0';
    
end generate tx3_NOT_generate;

tx3_generate: if(C_DUPLEX = 0) generate --Include collision cnts when 1  

-------------------------------------------------------------------------------
-- transmit collision retry down counter
-------------------------------------------------------------------------------
inst_colRetryCnt: entity xps_ethernetlite_v2_00_c.msh_cnt
  generic map
    (
    C_ADD_SUB_NOT => true,
    C_REG_WIDTH   => 5,
    C_RESET_VALUE => "00000"
    )
  
  port map
   (
    CK       => clk,
    RST      => txComboColRetryCntRst,
    Q        => collisionRetryCnt,
    Z        => open,
    LD       => "00000",
    AD       => "00001",
    LOAD     => '0',
    OP       => txColRetryCntEnbl
    );
            
txComboColRetryCntRst_n <= not(rst) and txColRetryCntRst_n;
txComboColRetryCntRst   <= not txComboColRetryCntRst_n;                             

------------------------------------------------------------------------------
-- transmit collision window nibble down counter
-------------------------------------------------------------------------------
inst_colWindowNibCnt: entity xps_ethernetlite_v2_00_c.msh_cnt
  generic  map
    (
    C_ADD_SUB_NOT => false,
    C_REG_WIDTH   => 8,
    C_RESET_VALUE => "10001111"
    )
  
  port map
   (
    CK       => clk,
    RST      => phy_tx_en_i_n,
    Q        => colWindowNibbleCnt,
    Z        => open,
    LD       => "10001111",
    AD       => "00000001",
    LOAD     => '0',
    OP       => txCntEnbl
    );   
    
phy_tx_en_i_n <= not(phy_tx_en_i);    

-------------------------------------------------------------------------------
-- jam transmit nibble down counter
-------------------------------------------------------------------------------
inst_jamTxNibCnt: entity xps_ethernetlite_v2_00_c.msh_cnt
  generic  map
    (
    C_ADD_SUB_NOT => false,
    C_REG_WIDTH   => 4,
    C_RESET_VALUE => "1001"
    )
  
  port map
   (
    CK       => clk,
    RST      => phy_tx_en_i_n,
    Q        => jamTxNibbleCnt,
    Z        => open,
    LD       => "1001",
    AD       => "0001",
    LOAD     => '0',
    OP       => jamTxComboNibCntEnbl
    );   
    
-------------------------------------------------------------------------------
-- tx collision back off counter
-------------------------------------------------------------------------------
  inst_bocnt: entity xps_ethernetlite_v2_00_c.bocntr
    port map
     (
      clk         => clk,
      clken       => txClkEn,
      rst         => rst,
      initBackoff => initBackoff,
      retryCnt    => collisionRetryCnt,
      backingOff  => backingOff
      ); 
      
end generate tx3_generate;   
     
 -------------------------------------------------------------------------------
 -- INT_tx_en_PROCESS
 -------------------------------------------------------------------------------
 -- This process assigns the outputs the phy enable
 -------------------------------------------------------------------------------
  INT_TX_EN_PROCESS: process (phy_tx_clk)
  begin  -- 
      if (phy_tx_clk'event and phy_tx_clk = '1') then
        if (rst = RESET_ACTIVE) then
          phy_tx_en_i <= '0';
          fifo_tx_en <= '0';
        else
          fifo_tx_en <= tx_en;
          phy_tx_en_i <= fifo_tx_en and tx_en;
        end if;
      end if;
  end process INT_TX_EN_PROCESS;            
  
  emac_tx_wr_mod <= emac_tx_wr or enblclear;
  tx_en_mod      <= '0' when enblclear = '1' else
                    tx_en;
  txChannelReset <= rst;
  txCntEnbl      <= txClkEn and phy_tx_en_i and not(not(full_half_n) and phy_col);

jamTxComboNibCntEnbl <= (phy_col or not(jamTxNibbleCnt(0) and
                         not(jamTxNibbleCnt(1)) and not(jamTxNibbleCnt(2)) and
                         jamTxNibbleCnt(3))) and pre_sfd_done and txclken and
                         not(full_half_n);
  
end imp;
