

//WARPnet Stuff

//Number of PPC clk cycles between stats updates
#define WARPNET_STATSPKT_PERIOD 60000000

//Index of OFDM PHY pkt buffer used for sending WARPnet Ethernet packets
#define WARPNET_SVR2NODE_PKTBUFFINDEX	0
#define WARPNET_NODE2SVR_PKTBUFFINDEX	31

#define WARPNET_CHANEST_PKTBUFFINDEX 2

#define WARPNET_NUMNODES 3

//The structs/defines below must match the definitions at the WARPNet server!

typedef struct {
	unsigned int val;
} warpnodeMailbox;


//Struct representing raw Ethernet header + packet contents
typedef struct {
	//Standard 14-byte Ethernet header (dest/src MAC addresses + EtherType)
	unsigned char dstAddr[6];
	unsigned char srcAddr[6];
	unsigned short ethType;
	
	//total number of bytes in pkt, including this header
	unsigned short pktLength;
	
	//number of warpnet structs that follow
	unsigned char numStructs;
	
	//Sequence number to match packets to ACKs
	unsigned char seqNum;
} warpnetEthernetPktHeader;

///Struct containing node statistics
typedef struct {
	unsigned char structType;
	unsigned char nodeID;
	unsigned char partnerID;
	///Reserved Byte
	unsigned char reserved0;
	///Used to track number of goodPackets
	unsigned int goodPackets;
	///Used to track number of partnerBadPackets
	unsigned int partnerBadPackets;
	///Used to track number of otherBadPackets
	unsigned int otherBadPackets;
	unsigned int rxBytes;
	unsigned int txBytes;
	///Time in clock cycles. Recall PPC405 clock freq is 240000000 HZ.
	unsigned long long time; 
} warpnodeStats;

typedef struct {
	unsigned char structType;
	unsigned char nodeID;
	unsigned char reserved0;
	unsigned char reserved1;
	
	unsigned int rxBits_total;
	unsigned int rxBits_errors;

	unsigned int rxPkts_good;
	unsigned int rxPkts_goodHeaderBadPayload;
	unsigned int rxPkts_badHeader;

	unsigned int txPkts;
	unsigned int txBits;
} warpnodeBERstats;

//Struct containing control parameters for the MAC/PHY
typedef struct {
	unsigned char structType;
	unsigned char nodeID;
	unsigned char modOrder;
	unsigned char txPower;
	unsigned char coding;
	unsigned char channel;
//	unsigned char mode;
} warpnodeControl;

//Struct containing control parameters for the MAC/PHY
typedef struct {
	unsigned char structType;
	unsigned char nodeID;
	unsigned char partnerID;
	unsigned char antennaID;
	unsigned char agc_RF;
	unsigned char agc_BB;
	unsigned char pktStatus;
	unsigned char reserved0;
	//64*4 bytes must follow in the actual Ethernet packet!
} warpnodeChanEst;

//Struct containing a command for a node
typedef struct {
	unsigned char structType;
	unsigned char nodeID;
	unsigned short cmdID;
	unsigned int cmdParam;
} warpnodeCommand;

//Struct containing just the node's ID (used to initialize node/server connection)
typedef struct {
	unsigned char structType;
	unsigned char nodeID;	
} warpnodeID;


///Struct containing traffic paramters for a node
typedef struct {
	unsigned char structType;
	///
	unsigned char nodeID;
	///Value of 1 denotes Transmit, Value of 0 denotes Receive
	unsigned char trafficMode; 
	///Reserved Byte
	unsigned char reserved0;
	///Interval between start of each packet (in usec)
	unsigned int txInterval; 
	///The length of the packet (in bytes)
	unsigned short txPacketLen;
	///Reserved Short
	unsigned short reserved1;
} warpnodeTraffic;

typedef struct {
	unsigned char structType;
	unsigned char nodeID;
	char msg[500];
} warpnetDebug;


//Struct Types for WARP Node <-> Server Packets
#define STRUCTID_NODESTATS 0x40
#define STRUCTID_NODEID 0x41
#define STRUCTID_NODECONTROL 0x42
#define STRUCTID_NODECOMMAND 0x43
#define STRUCTID_CHANEST 0x44
#define STRUCTID_NODETRAFFIC 0x45
#define STRUCTID_BERSTATS 0x46
#define STRUCTID_DEBUG 0x40

#define CHAN_EST_NUMBYTES 256

//Node command codes
#define NODECMD_RESETSTATS 0x61
#define NODECMD_REBOOT 0x62
#define NODECMD_NODEACK 0x63
#define NODECMD_START 0x66
#define NODECMD_STOP 0x67
#define NODECMD_REQUESTSTATS 0x68
#define NODECMD_REQUESTBER 0x69
#define NODECMD_RESETBER 0x70

//Initial node identification stuff
#define NODEID_UNKNOWN 0xFF

#define WARPNET_ETHTYPE_SVR2NODE 0x9090
#define WARPNET_ETHTYPE_NODE2SVR 0x9191
#define WARPNET_ETHTYPE_ARBTRAFFIC 0x9292

#define ETHTYPE_IPV4 0x0800
#define BROADCAST_NODE 0xFF

//TrafficMode Transmission definitions
#define TRANSMIT 1
#define RECEIVE 0


//Node<->Server mailbox bit masks
#define MBOX_SERVER_CONTROL_READY	0x0001 //0=server currently updating control struct
#define MBOX_NODE_CONTROL_READY		0x0002 //0=node currently reading control struct
#define MBOX_SERVER_CHANEST_READY	0x0004 //0=server currently reading chanEst structs
#define MBOX_NODE_CHANEST_READY		0x0008 //0=node currently updating chanEst structs

#define MBOX_CMD_RESET_STATS		0x1000 //1=server wants node to reset stats; node clears this bit
#define MBOX_CMD_MASK				0x1000 //Ones for each command bit

#define NODEID_READY 0x1234

//Put the WARPnet structs in the last 1KB of the shared BRAM
//#define WARPNET_STRUCTS_BASEADDR	(XPAR_XPS_BRAM_IF_CNTLR_1_HIGHADDR - 1023)
#define WARPNET_STRUCTS_BASEADDR	(XPAR_XPS_BRAM_IF_CNTLR_1_BASEADDR)

#define WARPNET_MAILBOX_CLEAR(addr, mask) (XIo_Out32((unsigned int)addr, XIo_In32((unsigned int)addr) & ~(mask)))
#define WARPNET_MAILBOX_SET(addr, mask) (XIo_Out32((unsigned int)addr, XIo_In32((unsigned int)addr) | (mask)))
#define WARPNET_MAILBOX_GET(addr, mask) ((XIo_In32((unsigned int)addr) & mask) > 0)











